@@ -1,14 +1,15 @@
 # datasets.py
 
-from geode.utilities import rasterize_polygon_layer, resample_dataset, tile_raster_pair
-from numpy import abs
+from geode.utilities import convert_labels_to_one_hots, rasterize_polygon_layer, resample_dataset, tile_raster_pair
+from numpy import abs, asarray, flip, float32, moveaxis, rot90
 from numpy.testing import assert_allclose
+from numpy.random import randint, shuffle
 from os import listdir, mkdir
 from os.path import isdir, join, splitext
 from osgeo import gdal, ogr
 
 
-class SemanticSegmentation:
+class Segmentation:
     """Defines a semantic segmentation dataset to be used in deep-learning models. Has methods to resample source
      imagery, to rasterize polygon layers, to generate training tiles, and to generate an iterator object
      for model training."""
@@ -305,7 +306,7 @@ class SemanticSegmentation:
 
         Args:
             output_path: the directory to hold the resampled imagery;
-            target_resolution: a tuple of the form (xRes, yRes) for target resolutions, in units of meters;
+            target_resolutions: a tuple of the form (xRes, yRes) for target resolutions, in units of meters;
             resample_algorithm: the method used for resampling (see gdalwarp documentation for more options);
             replace_source_dataset: whether to use the resampled imagery as the new source imagery;
             verbose: whether to print progress to the console.
@@ -361,8 +362,7 @@ class SemanticSegmentation:
         self.check_polygons()
 
     def training_generator(self, batch_size: int,
-                           use_tiles: bool = True,
-                           perform_one_hot: bool = True,
+                           perform_one_hot: bool = False,
                            n_classes: int = 2,
                            flip_vertically: bool = True,
                            rotate: bool = True,
@@ -371,7 +371,6 @@ class SemanticSegmentation:
 
         Args:
             batch_size: the number of tile pairs in each batch;
-            use_tiles: if true, uses the files at tiles_path; otherwise, it reads tiles from the source/label pairs;
             perform_one_hot: whether to do a one-hot encoding on the label tiles;
             n_classes: the number of label classes;
             flip_vertically: whether to randomly flip tile pairs vertically;
@@ -385,4 +384,47 @@ class SemanticSegmentation:
             Exception: if perform_one_hot is False and n_classes is not an integer greater than 1.
         """
 
-        # this method will be developed later
+        filenames = listdir(join(self.tiles_path, "imagery"))
+        train_ids = range(len(filenames))
+        while True:
+            shuffle(filenames)
+            for start in range(0, len(train_ids), batch_size):
+                imagery_batch = []
+                labels_batch = []
+                end = min(start + batch_size, len(filenames))
+                batch_ids = train_ids[start:end]
+
+                for ID in batch_ids:
+                    img = gdal.Open(join(self.tiles_path, "imagery", filenames[ID])).ReadAsArray()
+                    lbl = gdal.Open(join(self.tiles_path, "labels", filenames[ID])).ReadAsArray()
+
+                    # reshape img to channels-last
+                    img = moveaxis(img, 0, -1)
+
+                    # perform random rotation
+                    if rotate:
+                        k_rot = randint(0, 4)
+                        img = rot90(img, k=k_rot)
+                        lbl = rot90(lbl, k=k_rot)
+
+                    # perform random flip
+                    if flip_vertically and randint(0, 2) == 1:
+                        img = flip(img, axis=0)
+                        lbl = flip(lbl, axis=0)
+
+                    # perform a one-hot encoding of the labels
+                    if perform_one_hot:
+                        lbl = convert_labels_to_one_hots(lbl, n_classes)
+
+                    # rescale the input pixels
+                    img = img * scale_factor
+
+                    # append the rasters to a list
+                    imagery_batch.append(img)
+                    labels_batch.append(lbl)
+
+                # create an array of the full batch
+                imagery_batch = asarray(imagery_batch, dtype=float32)
+                labels_batch = asarray(labels_batch, dtype=float32)
+
+                yield imagery_batch, labels_batch
